from pypointgroup.gui.tools import GetIconPath
import os.path as pt
import shutil as sh
import os

__GROUPS = (
    r"1,~~ \left( C_1 \right)",
    r"\bar{1},~~ \left(C_i\right)",
    r"2,~~ \left(C_2\right)",
    r"m,~~ \left(C_s \right)",
    r"2/m,~~ \left(C_{2h}\right)",
    r"222,~~ \left(D_2\right)",
    r"mm2,~~ \left(C_{2v}\right)",
    r"mmm,~~ \left(D_{2h}\right)",
    r"4,~~ \left(C_4 \right)",
    r"\bar{4},~~ \left(S_4\right)",
    r"422,~~ \left( D_2 \right)",
    r"4/m,~~ \left( D_{4h} \right)",
    r"4mm,~~ \left( C_{4v} \right)",
    r"\bar{4}2m, ~~ \left( D_{2d} \right)",
    r"\frac{4}{m}mm, ~~ \left( D_{4h}\right)",
    r"3, ~~ \left( C_3\right)",
    r"\bar{3}, ~~ \left( C_{3i}\right)",
    r"32, ~~ \left( D_3 \right)",
    r"3m, ~~ \left( C_{3v} \right)",
    r"\bar{3}m, ~~ \left( D_{3d}\right)",
    r"6, ~~ \left( C_6 \right)",
    r"\bar{6}, ~~ \left( S_{6}, C_{3h} \right)",
    r"\bar{6}m2, ~~ \left( D_{3h} \right)",
    r"622, ~~ \left( D_{6}  \right)",
    r"6/m, ~~ \left( C_{6h} \right)",
    r"6mm, ~~ \left( C_{6v} \right)",
    r"\frac{6}{m}mm, ~~ \left( D_{6h} \right)",
    r"23, ~~ \left( T\right)",
    r"432, ~~ \left( O\right)",
    r"m\bar{3}, ~~ \left( T_h\right)",
    r"\bar{4}3m, ~~ \left( T_d\right)",
    r"m\bar{3}m, ~~ \left( O_h\right)",
)

IM_CMD = 'magick convert -density 300 -quality 90 "{infile}" "{outfile}"'
LATEX_CMD = 'xelatex -output-directory "{dir}" {infile}'
LATEX_DOC = r"""
\documentclass{standalone}
\usepackage{amsmath}
\usepackage{amsfonts}
\usepackage{amssymb}
\usepackage{graphicx}

\begin{document}
	$ %s $
\end{document}
"""

IMAGE_PATH = GetIconPath()
TMP_DIR = pt.join(IMAGE_PATH, 'tmp')

def convert_formula(formula, fname):

    texfile = pt.join(TMP_DIR, fname+".tex")
    imgfile = pt.join(IMAGE_PATH, fname+".png")
    pdffile = pt.join(TMP_DIR, fname+".pdf")

    with open(texfile,'wt', encoding='utf8') as f:
        f.write(LATEX_DOC % formula)

    os.system(LATEX_CMD.format(dir=TMP_DIR, infile=texfile))

    os.system(IM_CMD.format(infile=pdffile, outfile=imgfile))

    print("Created IMAG: ", imgfile)

def make_group_images():

    if not pt.exists(TMP_DIR):
        os.mkdir(TMP_DIR)

    for i, frm in enumerate(__GROUPS):
        fname = "pg_img_%d" % (i+1)
        convert_formula(frm, fname)

    sh.rmtree(TMP_DIR)

if __name__ == "__main__":
    make_group_images()


import typing

from PyQt5 import QtCore, QtGui
from PyQt5.QtWidgets import QOpenGLWidget, QWidget
from OpenGL.GL import *
from OpenGL.GLU import *
from pypointgroup.gl.gbuilder import GraphicsBuilder


class PGView(QOpenGLWidget):

    def __init__(self, parent=None) -> None:

        super().__init__(parent)
        self.x0 = 0
        self.y0 = 0
        self.dx = 0
        self.dy = 0
        self.flag = False
        self.GB = None
        self.ax = 0
        self.ay = 0
        self.az = 0
        self.x = 0
        self.y = 0
        self.z = -5
        self.group = []

    def setPointGroup(self, group:list):

        self.group = group

        self.ax = 0
        self.ay = 0
        self.az = 0
        self.x = 0
        self.y = 0
        self.z = -5
        self.flag = False
        self.repaint()


    def paintGL(self) -> None:

        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        glClearColor(0, 0, 0.8, 1)
        if not self.flag:
            glLoadIdentity()
            glTranslatef(self.x, self.y, self.z)
            glRotated(-self.ay, 1, 0, 0)
            glRotated(self.ax, 0, 1, 0)
            glRotated(self.az, 0, 0, 1)
            self.GB.Draw(self.group)
            glFinish()

    def initializeGL(self) -> None:

        coor = [0.0, 0.0, 1.0, 1.0]
        dirt = [0.0, 0.0, -1.0, 1.0]

        glEnable(GL_DEPTH_TEST) # разрешаем    тест  глубины
        glEnable(GL_LIGHTING) # разрешаем  работу  с  освещенностью
        glLightfv(GL_LIGHT0, GL_POSITION, coor)
        glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, dirt)
        glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, 1)
        glEnable(GL_LIGHT0) # включаем   источник  света    0

        self.GB = GraphicsBuilder()


    def resizeGL(self, w: int, h: int) -> None:

        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        gluPerspective(20.0, w / h, 1, 20.0)
        glViewport(0, 0, w, h)
        glMatrixMode(GL_MODELVIEW)
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        glClearColor(0, 0, 0.8, 1)

    def mouseMoveEvent(self, a0: QtGui.QMouseEvent) -> None:

        x = a0.x()
        y = a0.y()

        self.dx = x - self.x0
        self.dy = y - self.y0
        self.x0 = x
        self.y0 = y

        if a0.buttons() == QtCore.Qt.LeftButton:

            self.ax = self.ax + self.dx
            self.ay = self.ay - self.dy
            if self.ax > 360: self.ax = self.ax-360
            if self.ax < 0: self.ax = self.ax+360

            if self.ay > 360: self.ay = self.ay-360
            if self.ay < 0: self.ay = self.ay+360

            self.repaint()

        elif a0.buttons() == QtCore.Qt.RightButton:

            self.x = self.x + self.dx / 300
            self.z = self.z - self.dy / 100

            self.repaint()








# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'pgMainForm.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(820, 527)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.splitter = QtWidgets.QSplitter(self.centralwidget)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.glView = PGView(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(10)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.glView.sizePolicy().hasHeightForWidth())
        self.glView.setSizePolicy(sizePolicy)
        self.glView.setObjectName("glView")
        self.tabWidget = QtWidgets.QTabWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabWidget.sizePolicy().hasHeightForWidth())
        self.tabWidget.setSizePolicy(sizePolicy)
        self.tabWidget.setMinimumSize(QtCore.QSize(0, 0))
        self.tabWidget.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.tabWidget.setObjectName("tabWidget")
        self.tabGroups = QtWidgets.QWidget()
        self.tabGroups.setObjectName("tabGroups")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.tabGroups)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.lvGroups = QtWidgets.QListWidget(self.tabGroups)
        font = QtGui.QFont()
        font.setPointSize(14)
        self.lvGroups.setFont(font)
        self.lvGroups.setObjectName("lvGroups")
        self.gridLayout_2.addWidget(self.lvGroups, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tabGroups, "")
        self.tabOperators = QtWidgets.QWidget()
        self.tabOperators.setObjectName("tabOperators")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.tabOperators)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.lvOperators = PGOperatorListView(self.tabOperators)
        self.lvOperators.setMinimumSize(QtCore.QSize(0, 191))
        self.lvOperators.setObjectName("lvOperators")
        self.verticalLayout_4.addWidget(self.lvOperators)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.cbOperators = QtWidgets.QComboBox(self.tabOperators)
        self.cbOperators.setObjectName("cbOperators")
        self.verticalLayout_3.addWidget(self.cbOperators)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(self.tabOperators)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.tbOperName = QtWidgets.QLineEdit(self.tabOperators)
        self.tbOperName.setObjectName("tbOperName")
        self.verticalLayout.addWidget(self.tbOperName)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.tbQ11 = QtWidgets.QLineEdit(self.tabOperators)
        self.tbQ11.setObjectName("tbQ11")
        self.horizontalLayout.addWidget(self.tbQ11)
        self.tbQ12 = QtWidgets.QLineEdit(self.tabOperators)
        self.tbQ12.setObjectName("tbQ12")
        self.horizontalLayout.addWidget(self.tbQ12)
        self.tbQ13 = QtWidgets.QLineEdit(self.tabOperators)
        self.tbQ13.setObjectName("tbQ13")
        self.horizontalLayout.addWidget(self.tbQ13)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.tbQ21 = QtWidgets.QLineEdit(self.tabOperators)
        self.tbQ21.setObjectName("tbQ21")
        self.horizontalLayout_2.addWidget(self.tbQ21)
        self.tbQ22 = QtWidgets.QLineEdit(self.tabOperators)
        self.tbQ22.setObjectName("tbQ22")
        self.horizontalLayout_2.addWidget(self.tbQ22)
        self.tbQ23 = QtWidgets.QLineEdit(self.tabOperators)
        self.tbQ23.setObjectName("tbQ23")
        self.horizontalLayout_2.addWidget(self.tbQ23)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.tbQ31 = QtWidgets.QLineEdit(self.tabOperators)
        self.tbQ31.setObjectName("tbQ31")
        self.horizontalLayout_3.addWidget(self.tbQ31)
        self.tbQ32 = QtWidgets.QLineEdit(self.tabOperators)
        self.tbQ32.setObjectName("tbQ32")
        self.horizontalLayout_3.addWidget(self.tbQ32)
        self.tbQ33 = QtWidgets.QLineEdit(self.tabOperators)
        self.tbQ33.setObjectName("tbQ33")
        self.horizontalLayout_3.addWidget(self.tbQ33)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.verticalLayout_3.addLayout(self.verticalLayout_2)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem)
        self.bGenGroup = QtWidgets.QPushButton(self.tabOperators)
        self.bGenGroup.setEnabled(False)
        self.bGenGroup.setCheckable(False)
        self.bGenGroup.setObjectName("bGenGroup")
        self.horizontalLayout_4.addWidget(self.bGenGroup)
        self.bAddOperator = QtWidgets.QPushButton(self.tabOperators)
        self.bAddOperator.setObjectName("bAddOperator")
        self.horizontalLayout_4.addWidget(self.bAddOperator)
        self.verticalLayout_3.addLayout(self.horizontalLayout_4)
        self.verticalLayout_4.addLayout(self.verticalLayout_3)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem1)
        self.gridLayout_3.addLayout(self.verticalLayout_4, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tabOperators, "")
        self.tabCayley = QtWidgets.QWidget()
        self.tabCayley.setObjectName("tabCayley")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.tabCayley)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.tbViewer = QtWidgets.QTextBrowser(self.tabCayley)
        self.tbViewer.setTabStopWidth(200)
        self.tbViewer.setOpenLinks(False)
        self.tbViewer.setObjectName("tbViewer")
        self.gridLayout_6.addWidget(self.tbViewer, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tabCayley, "")
        self.gridLayout.addWidget(self.splitter, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 820, 19))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        self.menuTools = QtWidgets.QMenu(self.menubar)
        self.menuTools.setObjectName("menuTools")
        MainWindow.setMenuBar(self.menubar)
        self.actionSave_Image = QtWidgets.QAction(MainWindow)
        self.actionSave_Image.setObjectName("actionSave_Image")
        self.actionExit = QtWidgets.QAction(MainWindow)
        self.actionExit.setObjectName("actionExit")
        self.actionAbout_PointGroup = QtWidgets.QAction(MainWindow)
        self.actionAbout_PointGroup.setObjectName("actionAbout_PointGroup")
        self.actionCreate_shortcuts = QtWidgets.QAction(MainWindow)
        self.actionCreate_shortcuts.setObjectName("actionCreate_shortcuts")
        self.menuFile.addAction(self.actionSave_Image)
        self.menuFile.addAction(self.actionExit)
        self.menuHelp.addAction(self.actionAbout_PointGroup)
        self.menuTools.addAction(self.actionCreate_shortcuts)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuTools.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(2)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabGroups), _translate("MainWindow", "Groups"))
        self.label.setText(_translate("MainWindow", "Operator name:"))
        self.tbOperName.setText(_translate("MainWindow", "INV"))
        self.tbQ11.setText(_translate("MainWindow", "-1"))
        self.tbQ12.setText(_translate("MainWindow", "0"))
        self.tbQ13.setText(_translate("MainWindow", "0"))
        self.tbQ21.setText(_translate("MainWindow", "0"))
        self.tbQ22.setText(_translate("MainWindow", "-1"))
        self.tbQ23.setText(_translate("MainWindow", "0"))
        self.tbQ31.setText(_translate("MainWindow", "0"))
        self.tbQ32.setText(_translate("MainWindow", "0"))
        self.tbQ33.setText(_translate("MainWindow", "-1"))
        self.bGenGroup.setText(_translate("MainWindow", "Generate"))
        self.bAddOperator.setText(_translate("MainWindow", "Add"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabOperators), _translate("MainWindow", "Operators"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabCayley), _translate("MainWindow", "Cayley table"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.menuHelp.setTitle(_translate("MainWindow", "?"))
        self.menuTools.setTitle(_translate("MainWindow", "Tools"))
        self.actionSave_Image.setText(_translate("MainWindow", "Save Image..."))
        self.actionExit.setText(_translate("MainWindow", "Exit"))
        self.actionAbout_PointGroup.setText(_translate("MainWindow", "About PointGroup..."))
        self.actionCreate_shortcuts.setText(_translate("MainWindow", "Create shortcuts..."))
from pypointgroup.gl.pgoprview import PGOperatorListView
from pypointgroup.gl.pgview import PGView



from ._opers import POINT_OPERATORS
import numpy as np
from pypointgroup.core.Operators import Operator, SymmetryGroup

class Symmetry(dict):

    def __init__(self) -> None:
        super().__init__()
        self.update(POINT_OPERATORS)

    @property
    def Count(self):
        return len(self)

    def GetOperator(self, name:str):
        return Operator(name, self[name])

    def GetOperatorOfNumber(self, number: int):
        keys = list(self.keys())
        return Operator(keys[number], self[keys[number]])

    def GetName(self, oper)->str:

        if isinstance(oper, Operator): oper = oper.m

        for key, opr in self.items():
            if np.array_equal(oper, opr):
                return key

        return None

    def Add(self, name:str, Operator):
        self[name] = Operator

    def Delete(self, name:str):
        if name in self:
            del self[name]

    def GenGroup(self, gens:list):

        def __do(opr):
            name = self.GetName(opr)
            opr.name = name
            return name

        def __get_opr(opr):
            if isinstance(opr, str):
                return self.GetOperator(opr)
            elif isinstance(opr, Operator):
                return opr
            else:
                raise Exception('Invalid operator type: ', type(opr))

        g = SymmetryGroup([__get_opr(opr) for opr in gens])
        g.GenerateGroup()
        ng = [__do(opr) for opr in g]

        return ng, g



import typing

from PyQt5 import QtCore
from PyQt5.QtWidgets import QMenu, QAction
from pypointgroup.gui.tools import TryExcept

class ContextMenu(QMenu):

    def __init__(self, parent, ui) -> None:

        from pypointgroup.gl.pgoprview import PGOperatorListView
        from pypointgroup.gui.ui.pgMainForm_ui import Ui_MainWindow

        super().__init__(parent)

        self.ui:Ui_MainWindow = ui
        self.lv:PGOperatorListView = parent

        self.init_menu()

    def init_menu(self):
        self.addAction('Remove', self.OnRemoveItem)
        self.addAction('Clear', self.OnClear)
        self.addAction('Generate Group', self.OnGenerate)

    @TryExcept
    def OnRemoveItem(self):

        ixs = self.lv.selectedIndexes()

        if ixs:
            ix = ixs[0]
            i = ix.row()
            self.lv.removeOperator(i)

    @TryExcept
    def OnClear(self):
        self.lv.clearOperators()

    @TryExcept
    def OnGenerate(self):
        self.ui.bGenGroup.click()


import datetime
import os.path as pt

def new_version():

    path = pt.join(pt.dirname(__file__), 'Version.py')
    DATE = str(datetime.date.today())
    VERSION = "1.0.0"

    with open(path, 'rt') as f:
        for line in f.readlines():
            if line.startswith("VERSION"):
                VERSION = line[10:].strip().strip('"')

    a,b,c = VERSION.split('.')

    c = str(int(c)+1)

    print(f'Old version: {VERSION}')

    VERSION = f"{a}.{b}.{c}"

    print(f'New version: {VERSION}')
    print(f'Current date: {DATE}')

    with open(path, 'wt', encoding='utf8') as f:
        f.write(f"VERSION = \"{VERSION}\"\n")
        f.write(f"DATE = \"{DATE}\"\n")

    return VERSION, DATE

def new_conda(VERSION, DATE):

    path = pt.dirname(__file__)
    path, _ = pt.split(path)
    path = pt.join(path,'conda', 'meta.yaml')

    with open(path,'rt',encoding='utf8') as f:
        meta = [line for line in f.readlines()]

    for i,line in enumerate(meta):
        if line.startswith('  version:'):
            meta[i] = f"  version: {VERSION}\n"

    with open(path, 'wt', encoding='utf8') as f:
        f.writelines(meta)

    print("Creted ", path)

def main():
    VERSION, DATE = new_version()
    new_conda(VERSION, DATE)

if __name__ == '__main__':
    main()


from collections import Iterable

from PyQt5 import QtGui
from PyQt5.QtCore import QSize, QRect
from PyQt5.QtWidgets import QListWidget, QListWidgetItem, QMessageBox
from pypointgroup.core.Operators import Operator
from pypointgroup.gui.pgContextMenu import ContextMenu
from PyQt5.QtGui import QImage, QPixmap, QIcon, QPainter, QColor, QFont, QFontMetrics
from pypointgroup.gui.tools import TryExcept, LoadIcon


class PGOperatorListView(QListWidget):

    ICON_SIZE = 64

    def __init__(self, parent) -> None:
        super().__init__(parent)
        self.pg = list()
        self.init_control()
        self.font_size = None

    def init_control(self):
        self.setViewMode(QListWidget.IconMode)
        self.setSpacing(10)
        self.setIconSize(QSize(self.ICON_SIZE, self.ICON_SIZE))
        self.setResizeMode(QListWidget.Adjust)

    def setUI(self, ui):

        from pypointgroup.gui.ui.pgMainForm_ui import Ui_MainWindow
        self.ui: Ui_MainWindow = ui

    def addOperator(self, opr:Operator):

        self.pg.append(opr)
        ico = self.getIconOpr(opr) # LoadIcon('Icon.ico')
        item = QListWidgetItem(ico, opr.name)
        self.addItem(item)
        self.ui.bGenGroup.setEnabled(self.count() > 0)

    def setOperators(self, oprs: Iterable):

        self.pg.clear()
        self.clear()

        for opr in oprs:
            self.addOperator(opr)

    def removeOperator(self, index:int):
        self.pg.pop(index)
        self.takeItem(index)
        self.ui.bGenGroup.setEnabled(self.count() > 0)

    def clearOperators(self):
        self.pg.clear()
        self.clear()
        self.ui.bGenGroup.setEnabled(self.count() > 0)

    def getOperators(self):
        return self.pg

    def contextMenuEvent(self, a0: QtGui.QContextMenuEvent) -> None:
        menu = ContextMenu(self, self.ui)
        menu.exec(a0.globalPos())

    def getIconOpr(self, opr:Operator):

        bmp = QPixmap(self.ICON_SIZE,self.ICON_SIZE)
        bmp.fill(QColor('white'))
        cv = QPainter()
        cv.begin(bmp)

        cv.setPen(QColor('blue'))
        cv.drawRect(1,1,self.ICON_SIZE-2, self.ICON_SIZE-2)

        font = cv.font()
        font.setBold(True)

        if not self.font_size:

            font.setPointSize(10)
            fm = QFontMetrics(font)
            w10 = fm.width(' -1  -1  -1 ')

            font.setPointSize(12)
            fm = QFontMetrics(font)
            w12 = fm.width(' -1  -1  -1 ')

            k = (w12 - w10) / 2
            b = w10 - k*10
            s = (self.ICON_SIZE - b) / k
            self.font_size = int(s)

        font.setPointSize(self.font_size)
        cv.setFont(font)

        cv.setPen(QColor('red'))

        for i, row in enumerate(opr.m):
            cv.drawText(0, 14*(i+1)+7, " ".join(["%3d" % x for x in row]))

        cv.end()

        return QIcon(bmp)







import numpy as np
from collections.abc import Iterable
from itertools import product
from io import StringIO

class Operator:

    def __init__(self, name:str=None, m:np.ndarray=None):
        self.m = m
        self.name = name
        self.hash = None

    def __str__(self):
        return self.name

    def __eq__(self, other):
        return np.array_equal(self.m, other.m)

    def __ne__(self, other):
        return not np.array_equal(self.m, other.m)

    def __mul__(self, other):
        return Operator(name = "%s %s" % (self.name, other.name), m = np.matmul(self.m, other.m))

    def __hash__(self):
        if self.hash is None:
            p = np.array([3**i for i in range(9)])
            self.hash = int(((self.m.reshape(9) + 1) * p).sum())
        return self.hash

    def toHTML(self, name=None)->str:
        if not name: name = self.name
        tab = f"""
        <table>
        <tr><td colspan=3>{name}</td></tr>
        <tr> <td>{self.m[0,0]}</td> <td>{self.m[0,1]}</td> <td>{self.m[0,2]}</td></tr>
        <tr> <td>{self.m[1,0]}</td> <td>{self.m[1,1]}</td> <td>{self.m[1,2]}</td></tr>
        <tr> <td>{self.m[2,0]}</td> <td>{self.m[2,1]}</td> <td>{self.m[2,2]}</td></tr>
        </table>
        """
        return tab
#========================================================================

class SymmetryGroup(set):

    def __init__(self, items=None):

        super().__init__()

        if items:
            if isinstance(items, Iterable):
                self.update(items)
            elif isinstance(items, Operator):
                self.add(items)

    def GenerateGroup(self):

        flag = True
        while flag:
            new = [o1 * o2 for o1, o2 in product(self,self)]
            lx1 = len(self)
            self.update(new)
            flag = len(self) > lx1

            if lx1 > 48:
                raise Exception("Group generators are set incorrectly!")

        return self

    def toHTML(self)->str:

        buf = StringIO()

        buf.write("<html><head></head><body>")

        buf.write("<h1 align=center>Cayley table</h1>")

        buf.write(self.getCayleyTabAsHTML())

        buf.write("\n<hr>\n")

        buf.write("<h1 align=center>Operators</h1>")

        for i, opr in enumerate(self):
            buf.write(opr.toHTML(name=f"Q<sub>{i+1}<sub>"))
            #buf.write("<br>")

        buf.write("</body></html>")

        return buf.getvalue()

    def getCayleyTabAsHTML(self)->str:

        data, n = self.getCayleyTab()

        buf = StringIO()

        buf.write(f'<table cellspacing="2" border="1" cellpadding="5" width="{n*60}">\n')

        for row in data:
            buf.write("<tr>")
            for item in row:
                buf.write(f"<td>{item}</td>")
            buf.write("</tr>\n")

        buf.write("</table>")

        return buf.getvalue()

    def getCayleyTab(self):

        opers = list(self)
        n = len(opers)
        data = [["-" for j in range(n+1)] for i in range(n+1)]

        for i, o in enumerate(opers):
            data[0][i+1] = data[i+1][0] = f"<b>Q<sub>{i+1}<sub></b>"#o.name


        for i, o1 in enumerate(opers):
            for j, o2 in enumerate(opers):
                o = o1*o2
                ix = opers.index(o)
                data[i+1][j+1] = f"Q<sub>{ix+1}<sub>"#opers[ix].name

        return data, n


from OpenGL.GL import *
from pypointgroup.gl._gconsts import *
from OpenGL.GLU import *
from PyQt5.Qt import QImage
from PyQt5.QtOpenGL import QGLWidget
import numpy as np
import os.path as pt
from pypointgroup.gui.tools import GetIconPath


def glRotate(Angle, axe):
     glRotated(Angle, *axe)


class GraphicsBuilder:
    
    IMAGE_PATH = None
    
    def __init__(self) -> None:
        self.flag:bool = True
        self.Gex:bool = False
        self.textures = dict()
        self.LoadTextures()

    def _load_texture(self, file, id):

        path = pt.join(self.IMAGE_PATH, file)     
        img = QImage()
        if img.load(path):
            self.textures[id] = QGLWidget.convertToGLFormat(img)
            print(f'Texture {path} loaded...[OK]')
        else:
            print(f'Texture {path} loaded...[FAIL]')


    def LoadTextures(self):
        self.flag = True
        self.IMAGE_PATH = GetIconPath()

        glEnable(GL_TEXTURE_2D)
        self.tex = glGenTextures(1)

        self._load_texture('2.BMP', a_2)
        self._load_texture('3.BMP', a_3)
        self._load_texture('4.BMP', a_4)
        self._load_texture('6.BMP', a_6)
        self._load_texture('4I.BMP', a_4i)
        self._load_texture('3I.BMP', a_3i)
        self._load_texture('6I.BMP', a_6i)

        self._load_texture('2SP.BMP', a_2sg)
        self._load_texture('21SPv.bmp', a_21sgv)
        self._load_texture('21SPh.bmp', a_21sgh)

        self._load_texture('M.bmp',  m_m)
        self._load_texture('Ma.bmp', m_a)
        self._load_texture('Mb.bmp', m_b)
        self._load_texture('Md.bmp', m_d)

    def GetCub(self, x,y,z,a):

        glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, Axies4_Color)
        a = a / 2
        glBegin(GL_QUADS)
        glNormal3f(-1, 0, 0)
        glTexCoord2f(0, 0)
        glVertex3f(x + a, y - a, z + a)
        glTexCoord2f(1, 0)
        glVertex3f(x + a, y + a, z + a)
        glTexCoord2f(1, 1)
        glVertex3f(x + a, y + a, z - a)
        glTexCoord2f(0, 1)
        glVertex3f(x + a, y - a, z - a)

        glNormal3f(-1, 0, 0)
        glTexCoord2f(0, 0)
        glVertex3f(x - a, y - a, z + a)
        glTexCoord2f(1, 0)
        glVertex3f(x - a, y + a, z + a)
        glTexCoord2f(1, 1)
        glVertex3f(x - a, y + a, z - a)
        glTexCoord2f(0, 1)
        glVertex3f(x - a, y - a, z - a)

        glNormal3f(0, -1, 0)
        glTexCoord2f(0, 0)
        glVertex3f(x + a, y + a, z + a)
        glTexCoord2f(1, 0)
        glVertex3f(x - a, y + a, z + a)
        glTexCoord2f(1, 1)
        glVertex3f(x - a, y + a, z - a)
        glTexCoord2f(0, 1)
        glVertex3f(x + a, y + a, z - a)

        glNormal3f(0, -1, 0)
        glTexCoord2f(0, 0)
        glVertex3f(x + a, y - a, z + a)
        glTexCoord2f(1, 0)
        glVertex3f(x - a, y - a, z + a)
        glTexCoord2f(1, 1)
        glVertex3f(x - a, y - a, z - a)
        glTexCoord2f(0, 1)
        glVertex3f(x + a, y - a, z - a)

        glNormal3f(0, 0, 1)
        glTexCoord2f(0, 0)
        glVertex3f(x + a, y - a, z + a)
        glTexCoord2f(1, 0)
        glVertex3f(x + a, y + a, z + a)
        glTexCoord2f(1, 1)
        glVertex3f(x - a, y + a, z + a)
        glTexCoord2f(0, 1)
        glVertex3f(x - a, y - a, z + a)

        glNormal3f(0, 0, 1)
        glTexCoord2f(0, 0)
        glVertex3f(x + a, y - a, z - a)
        glTexCoord2f(1, 0)
        glVertex3f(x + a, y + a, z - a)
        glTexCoord2f(1, 1)
        glVertex3f(x - a, y + a, z - a)
        glTexCoord2f(0, 1)
        glVertex3f(x - a, y - a, z - a)

        glEnd()

    def GetInvers(self):
        q1 = gluNewQuadric()
        glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, Invers_Color)
        gluSphere(q1, SizeAxis, 10, 10)

    def GetAxis(self, Orientation, TypAxe):

        glPushMatrix()
        glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, m_Color)

        if (TypAxe==a_3) or (TypAxe==a_3i):
            if Orientation == e_y: glRotated(90, 1, 0, 0)
            elif Orientation == e_x: glRotated(90, 0, 1, 0)
            elif Orientation == e_xy: glRotated(90, 1, 1, 0)
            elif Orientation == e_cub_1: glRotated(54, 1, 1, 0)
            elif Orientation == e_cub_2: glRotated(-54, 1, 1, 0)
            elif Orientation == e_cub_3: glRotated(54, -1, 1, 0)
            elif Orientation == e_cub_4: glRotated(-54, -1, 1, 0)
            elif Orientation == e__xy: glRotated(90, -1,1,0)
            elif Orientation == e_x_hex: glRotate(90, Axe_y)
            elif Orientation == e_y_hex: glRotate(90, Axe_u)
            elif Orientation == e_xy_Hex: glRotate(90, Axe__u)

            elif Orientation == e__xy_Hex: glRotate(90, Axe_x)
            elif Orientation == e_xy__Hex: glRotate(90, Axe_u2)
            elif Orientation == e__xy__Hex: glRotate(90, Axe_u3)
        else:
            if Orientation == e_y: glRotated(90, 1, 0, 0)
            elif Orientation == e_x: glRotated(90, 0, 1, 0)
            elif Orientation == e_xy: glRotated(90, 1, 1, 0)
            elif Orientation == e_cub_1: glRotated(45, 1, 0, 0)
            elif Orientation == e_cub_2: glRotated(-45, 1, 0, 0)
            elif Orientation == e_cub_3: glRotated(45, 0, 1, 0)
            elif Orientation == e_cub_4: glRotated(-45, 0, 1, 0)
            elif Orientation == e__xy: glRotated(90, -1,1,0)
            elif Orientation == e_x_hex: glRotate(90, Axe_x)
            elif Orientation == e_y_hex: glRotate(90, Axe_u2)
            elif Orientation == e_xy_Hex: glRotate(90, Axe_u3)

            elif Orientation == e__xy_Hex: glRotate(90, Axe_y)
            elif Orientation == e_xy__Hex: glRotate(90, Axe_u)
            elif Orientation == e__xy__Hex: glRotate(90, Axe__u)

        glBegin(GL_LINES)
        glVertex3f(0, 0, LenAxis)
        glVertex3f(0, 0, -LenAxis)
        glEnd()
        self.GetAxisSymbol(0, 0, LenAxis, TypAxe)
        self.GetAxisSymbol(0, 0, -LenAxis, TypAxe)
        glPopMatrix()


    def GetAxisSymbol(self, x,y,z, TypAxe):
        self.SetTexture(TypAxe)
        self.GetCub(x, y, z, SizeAxis)


    def Getm(self, AxiTipe):
        self.GetDisk(RadiusSpher, 30, AxiTipe)

    def GetDisk(self, R, n, Orientation):

        glPushMatrix()
        glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, m_Color)

        if Orientation == e_y: glRotated(90, 1, 0, 0)
        elif Orientation == e_x: glRotated(90, 0, 1, 0)
        elif Orientation == e_xy: glRotated(90, 1, 1, 0)
        elif Orientation == e_cub_1: glRotated(45, 1, 0, 0)
        elif Orientation == e_cub_2: glRotated(-45, 1, 0, 0)
        elif Orientation == e_cub_3: glRotated(45, 0, 1, 0)
        elif Orientation == e_cub_4: glRotated(-45, 0, 1, 0)
        elif Orientation == e__xy: glRotate(90, Axe__xy)

        elif Orientation == e_x_hex: glRotate(90, Axe_y)
        elif Orientation == e_y_hex: glRotate(90, Axe_u)
        elif Orientation == e_xy_Hex: glRotate(90, Axe__u)#

        elif Orientation == e__xy_Hex: glRotate(90, Axe_x)
        elif Orientation == e_xy__Hex: glRotate(90, Axe_u3) #
        elif Orientation == e__xy__Hex: glRotate(90, Axe_u2)

        glBegin(GL_POLYGON)
        glNormal3f(0, 0, -1)
        ph = np.linspace(0, 2*np.pi, n)
        X = R * np.sin(ph)
        Y = R * np.cos(ph)
        z = 0
        for x,y in zip(X,Y):
            glVertex3f(x, y, z)
        glEnd()
        glPopMatrix()

    def PointGroup(self, oList):

        self.Selector(oList)

        for opr in oList:
            if opr == '2Z':  self.GetAxis(e_z, a_2)
            elif opr == '2X':  self.GetAxis(e_x, a_2)
            elif opr == '2Y': self.GetAxis(e_y, a_2)
            elif opr == '2Y Hex':  self.GetAxis(e_y_hex, a_2)
            elif opr == '2X Hex': self.GetAxis(e_x_hex, a_2)
            elif opr == '2XY':
                if self.Gex : 
                    self.GetAxis(e__xy_Hex, a_2)
                else:
                    self.GetAxis(e_xy, a_2)

            elif opr == '2U' : self.GetAxis(e_y_hex, a_2)

            elif opr == '2X-Y' : self.GetAxis(e_xy__Hex, a_2)
            elif opr == '2-XY' : self.GetAxis(e__xy__Hex, a_2)

            elif opr == '2-X-Y':
                if self.Gex:
                    self.GetAxis(e_xy_Hex, a_2)
                else:
                    self.GetAxis(e__xy, a_2)

            elif opr == '2 Cub1' : self.GetAxis(e_cub_1, a_2)
            elif opr == '2 Cub2' : self.GetAxis(e_cub_2, a_2)
            elif opr == '2 Cub3' : self.GetAxis(e_cub_3, a_2)
            elif opr == '2 Cub4' : self.GetAxis(e_cub_4, a_2)

            elif opr == '3Z' : self.GetAxis(e_z, a_3)
            elif opr == '3XYZ' : self.GetAxis(e_cub_1, a_3)
            elif opr == '3XYZ3' : self.GetAxis(e_cub_2, a_3)
            elif opr == '3XYZ2' : self.GetAxis(e_cub_3, a_3)
            elif opr == '3XYZ1' : self.GetAxis(e_cub_4, a_3)
            elif opr == '3Zi' : self.GetAxis(e_z, a_3i)

            elif opr == '3XYZ1i' : self.GetAxis(e_cub_1, a_3i)
            elif opr == '3XYZ2i' : self.GetAxis(e_cub_2, a_3i)
            elif opr == '3XYZ3i' : self.GetAxis(e_cub_3, a_3i)
            elif opr == '3XYZ4i' : self.GetAxis(e_cub_4, a_3i)

            elif opr == '4Z' : self.GetAxis(e_z, a_4)
            elif opr == '4X' : self.GetAxis(e_x, a_4)
            elif opr == '4Y' : self.GetAxis(e_y, a_4)
            elif opr == '4Zi' : self.GetAxis(e_z, a_4i)
            elif opr == '4Xi' : self.GetAxis(e_x, a_4i)
            elif opr == '4Yi' : self.GetAxis(e_y, a_4i)

            elif opr == '6Z' : self. GetAxis(e_z, a_6)
            elif opr == '6Zi' : self. GetAxis(e_z, a_6i)

            elif opr == 'MZ' : self.Getm(e_z)
            elif opr == 'MX' : self.Getm(e_x)
            elif opr == 'MY' : self.Getm(e_y)

            elif opr == 'MXY':
                if self.Gex:
                    self.Getm(e_xy_Hex)
                else:
                    self.Getm(e_xy)

            elif opr == 'M-X-Y':
                if self.Gex :
                    self.Getm(e__xy_Hex)
                else:
                    self.Getm(e__xy)
            elif opr == 'MX Hex' : self.Getm(e_x_hex)
            elif opr == 'MY Hex' : self.Getm(e_y_hex)

            elif opr == 'M Cub1' : self.Getm(e_cub_1)
            elif opr == 'M Cub2' : self.Getm(e_cub_2)
            elif opr == 'M Cub3' : self.Getm(e_cub_3)
            elif opr == 'M Cub4' : self.Getm(e_cub_4)
            elif opr == 'MU' : self.Getm(e_xy__Hex)
            elif opr == 'M-U' : self.Getm(e__xy__Hex)

            elif opr == 'IN' : self.GetInvers()


    def Selector(self, oList:list):

        kill = lambda x:oList.remove(x) if x in oList else None

        if 'TR' in oList: oList.remove('TR')
        self.Gex = False

        if '4Z' in oList:
            kill('4Zi')
            kill('2Z')

        if '4X' in oList:
            kill('2X')
            kill('4Xi')

        if '4Y' in oList:
            kill('2Y')
            kill('4Yi')

        if '4Zi' in oList: kill('2Z')
        if '4Xi' in oList: kill('2X')
        if '4Yi' in oList: kill('2Y')

        if '3Z' in oList:
            kill('3Z2')
            self.Gex = True

        if '3XYZ1i' in oList:
            kill('3XYZ')
            kill('3XYZ1')
            kill('3XYZ2')
            kill('3XYZ3')
            kill('3XYZ4')

        if '3Zi' in oList:
            kill('3Z')
            kill('6S4')
            self.Gex = True

        if '6Zi' in oList:
            self.Gex = True

        if '6Z' in oList:
            kill('6Z5')
            kill('2Z')
            kill('3Z')
            self.Gex = True

    def SetTexture(self, id:int):

        img:QImage = self.textures[id]
        bits = img.bits()
        bits.setsize(img.byteCount())
        ptr = bits.asstring()

        glBindTexture(GL_TEXTURE_2D, self.tex)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 64, 64, 0, GL_RGBA, GL_UNSIGNED_BYTE, ptr)

    def Draw(self, oList:list):

        #if self.flag:
        #glNewList(1, GL_COMPILE_AND_EXECUTE)
        self.PointGroup(oList)
        #glEndList()
        #self.flag = False
        #else:
        #    glCallList(1)

if __name__ == "__main__":
    obj = GraphicsBuilder()


from PyQt5.QtCore import QSize
from pypointgroup.Version import VERSION, DATE
from .ui.pgMainForm_ui import Ui_MainWindow
from PyQt5.QtWidgets import QMainWindow, QMessageBox, QFileDialog, QListWidgetItem, QListWidget
from PyQt5.QtGui import QImage, QIcon
from pypointgroup.core.pgroupsgen import POINT_GROUPS_GENETATORS, POINT_GROUPS_LIST
from pypointgroup.core.symmetry import Symmetry, Operator, np
from pypointgroup.gui.tools import TryExcept, LoadIcon, GetIconPath, CreateShortCuts
from datetime import date


class PGMainForm(QMainWindow):

    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.init_controls()
        self.init_events()

    @TryExcept
    def init_events(self):
        gui = self.ui
        gui.lvGroups.clicked.connect(self.OnGroupClick)
        gui.actionExit.triggered.connect(self.OnClose)
        gui.actionSave_Image.triggered.connect(self.OnSaveImage)
        gui.actionAbout_PointGroup.triggered.connect(self.OnAbout)
        gui.actionCreate_shortcuts.triggered.connect(self.OnCreateShortCuts)
        gui.cbOperators.currentTextChanged.connect(self.OnChangeOperator)
        gui.bAddOperator.clicked.connect(self.OnAddOperator)
        gui.bGenGroup.clicked.connect(self.OnGenerateClick)

    @TryExcept
    def init_controls(self):

        self.setWindowTitle('PointGroup')
        icon = LoadIcon('Icon.ico')
        self.setWindowIcon(icon)

        gui = self.ui
        self.loadPointGroups()

        self.sym = Symmetry()
        gui.cbOperators.addItems(self.sym.keys())

        self.QM = (
            (gui.tbQ11, gui.tbQ12, gui.tbQ13),
            (gui.tbQ21, gui.tbQ22, gui.tbQ23),
            (gui.tbQ31, gui.tbQ32, gui.tbQ33),
        )

        gui.lvOperators.setUI(gui)

        gui.tabWidget.setCurrentIndex(0)

    @TryExcept
    def loadPointGroups(self):

        gui = self.ui

        gui.lvGroups.setSpacing(10)
        gui.lvGroups.setIconSize(QSize(150, 100))
        gui.lvGroups.setResizeMode(QListWidget.Adjust)

        try:
            for i in range(1,33):
                img_path = GetIconPath("pg_img_%d.png" % i)
                ico = QIcon(img_path)
                item = QListWidgetItem(ico,'#%d' % i)
                gui.lvGroups.addItem(item)
                print(f"Image {img_path} loaded ... [OK]")
        except Exception as ex:
            print("WARNING: Can't load groups: ", str(ex))
            gui.lvGroups.clear()
            gui.lvGroups.addItems(POINT_GROUPS_LIST)

    @TryExcept
    def OnGroupClick(self):

        gui = self.ui
        ix = gui.lvGroups.currentIndex()
        if ix:
            i = ix.row()
            ng, g = self.sym.GenGroup(POINT_GROUPS_GENETATORS[i])
            gui.tbViewer.setHtml(g.toHTML())
            gui.glView.setPointGroup(ng)

    def OnClose(self):
        self.close()

    @TryExcept
    def OnSaveImage(self):
        img = self.ui.glView.grabFramebuffer()
        path, fl = QFileDialog.getSaveFileName(self,
                    "Save Image", filter='JPG files (*.jpg);;All files (*.*)')
        if path:
            img.save(path)

    def OnAbout(self):
        text = f"""
PointGroup v. {VERSION}
(C) 2020 - {date.today().year}. Nikolay V. Somov
Build: {DATE}
e-mail: somov@phys.unn.ru
        """
        QMessageBox.about(self, "About PointGroup", text)

    @TryExcept
    def OnChangeOperator(self):
        gui = self.ui
        opr = gui.cbOperators.currentText()
        q = self.sym.GetOperator(opr)

        gui.tbOperName.setText(q.name)

        for i, row in enumerate(self.QM):
            for j, tb in enumerate(row):
                tb.setText(str(q.m[i,j]))

    @TryExcept
    def OnAddOperator(self):
        gui = self.ui
        m = [[int(tb.text()) for tb in row] for row in self.QM]
        opr = Operator(gui.tbOperName.text(), np.array(m))
        gui.lvOperators.addOperator(opr)

    @TryExcept
    def OnGenerateClick(self):
        gui = self.ui
        ng, g = self.sym.GenGroup(gui.lvOperators.getOperators())
        gui.lvOperators.setOperators(g)
        gui.glView.setPointGroup(ng)
        gui.tbViewer.setHtml(g.toHTML())

    @TryExcept
    def OnCreateShortCuts(self):
        CreateShortCuts(self)


from PyQt5.QtWidgets import QMessageBox
from PyQt5.QtGui import QIcon
import os.path as pt
import sys

def TryExcept(target):

    def __do_target(self):
        try:
            target(self)
        except Exception as ex:
            QMessageBox.critical(self, 'Error', str(ex), QMessageBox.Ok)

    return __do_target

def GetIconPath(image_file:str=None):

    if image_file:
        return pt.join(*pt.split(pt.dirname(__file__))[:-1], "img", image_file)
    else:
        return pt.join(*pt.split(pt.dirname(__file__))[:-1], "img")

def LoadIcon(icon_file:str):
    return QIcon(GetIconPath(icon_file))

def CreateShortCuts(parent):

    try:
        from pyshortcuts import make_shortcut
    except:
        raise Exception("The system is missing package PyShortCuts. Please install it and try again: pip install pyshortcuts")

    bindir = 'bin'
    name = 'pypointgroup'
    if sys.platform.startswith('win'):
        bindir = 'Scripts'
        name = name + '.exe'

    run_path = pt.join(sys.prefix, bindir, name)

    if not pt.exists(run_path):

        p_path, _ = pt.split(pt.dirname(__file__))
        p_path, _ = pt.split(p_path)
        run_path = pt.join(p_path, "pypointgroup.py")

    if pt.exists(run_path):
        print('[OK!]')
        QMessageBox.information(parent, 'Creating shortcuts', "OK", QMessageBox.Ok)

        make_shortcut(script=run_path, name='PyPointGroup', icon=GetIconPath('Icon.ico'),
                          desktop=True, startmenu=True, terminal=False)

        QMessageBox.information(parent, 'Creating shortcuts',
                                "Shortcuts have been successfully created!", QMessageBox.Ok)
    else:
        raise Exception("Can't create shortcuts. Start script not found! [%s]" % run_path)


from setuptools import setup, find_packages
from pypointgroup import VERSION
#-----------------------------------------------------------------------------------------

def load_description():
    with open('README.md','r') as f:
        text = f.read()
    return text

#-----------------------------------------------------------------------------------------
#
#-----------------------------------------------------------------------------------------

setup(
    name='PyPointGroup',
    version=VERSION,
    python_requires='>3.4',
    packages=['pypointgroup', 'pypointgroup.gl', 'pypointgroup.gui', 'pypointgroup.gui.ui', 'pypointgroup.core'],
    url='https://github.com/nik0380/pypointgroup',
    license='MIT',
    author='Nikolay Somov',
    author_email='somov@phys.unn.ru',
    description='Point group symmetry study software',
    long_description=load_description(),
    long_description_content_type='text/markdown',
    keywords=['symmetry','group','crystallography','xray'],
    install_requires=["PyQt5", "numpy","PyOpenGL"],
    include_package_data=True,
    entry_points={'gui_scripts': ['pypointgroup=pypointgroup:main',],},
)



from pypointgroup.gui.PGMainForm import PGMainForm, LoadIcon
from PyQt5.QtWidgets import QApplication
import sys

def main():

    if sys.platform == 'win32':
        try:
            import ctypes
            myappid = 'unn.pypointgroup.viewer.1'
            ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)
        except:
            pass

    app = QApplication(sys.argv)
    form = PGMainForm()
    app.setWindowIcon(LoadIcon('Icon.ico'))
    form.show()
    app.exec_()

if __name__ == "__main__":

    main()